--
-- This file contains operations on bags/items that are not really part of 
-- Baggins core functionality, such as compressing your inventory, splitting
-- items, etc.
--

local Baggins = Baggins
local pt = Baggins.pt
local L = Baggins.L
local sieve = Baggins.sieve
local dewdrop = Baggins.dewdrop


local bankBags = { BANK_CONTAINER }
for i=NUM_BAG_SLOTS+1, NUM_BAG_SLOTS+NUM_BANKBAGSLOTS do
	tinsert(bankBags, i);
end

local charBags = {}
for i=0, NUM_BAG_SLOTS do
	tinsert(charBags, i);
end


----------------------
-- Compress stacks  --
----------------------


local incompleteSlots = {}	-- pairs of itemlink = bag*1000+slot, e.g. "foo" = 207 (bag 2 slot 7)
local compressLoopProtect = 0

function Baggins:CompressBags(bank,testonly)
	local bags = bank and bankBags or charBags;
	
	if not testonly and CursorHasItem() then 
		if not self.compress_target_bag then
			return true; -- done, can't do more =(
		end
		PickupContainerItem(self.compress_target_bag, self.compress_target_slot);
		self:ScheduleEvent("Baggins_CompressBags", self.CompressBags, 0.1, self, bank);
		return;
	end
	self.compress_target_bag = nil;
	
	for k,v in pairs(incompleteSlots) do
		incompleteSlots[k] = nil;
	end
	
	for _,bag in ipairs(bags) do
		for slot=1,(GetContainerNumSlots(bag) or -1) do
			local texture, itemCount, locked, quality, readable = GetContainerItemInfo(bag, slot);
			local link = GetContainerItemLink(bag, slot)
			if link then
				local sName, sLink, iRarity, iLevel, iMinLevel, sType, sSubType, iMaxStack = GetItemInfo(link);
				if itemCount < iMaxStack then
					local itemid = link:match("item:(-?[%d]+):");
					if(incompleteSlots[itemid]) then	-- see if we have an incomplete stack of this sitting around
						if testonly then return true; end	-- Yup, we've got something that needs compressing!
						compressLoopProtect = compressLoopProtect + 1;
						if compressLoopProtect > 20 then compressLoopProtect=0; return; end
						
						PickupContainerItem(bag,slot);
						self.compress_target_bag = floor(incompleteSlots[itemid]/1000);
						self.compress_target_slot = incompleteSlots[itemid]%1000;
						self:ScheduleEvent("Baggins_CompressBags", self.CompressBags, 0.1, self, bank);
						return;
					else
						incompleteSlots[itemid] = bag*1000 + slot;	-- remember that there's an incomplete stack of this item sitting around
					end
				end
			end				
		end
	end
	
	if not testonly then
		compressLoopProtect = 0
	end
	
end


local lastCanCompressBags = "maybe";
local lastCanCompressBank = "maybe";

function Baggins:RecheckCompress(delay)
	if delay then
		self:ScheduleEvent("Baggins_RecheckCompress", self.RecheckCompress, delay, self, nil);
		return;
	end
	local b
	b = self:CompressBags(false, true);	-- false=bags, true=testonly
	if b ~= lastCanCompressBags then
		lastCanCompressBags = b;
		self:FireSignal("Baggins_CanCompress", false, b);		-- false=bags
	end
	b = self:CompressBags(true, true);	-- true=bank, true=testonly
	if b ~= lastCanCompressBank then
		lastCanCompressBank = b;
		self:FireSignal("Baggins_CanCompress", true, b);  -- true=bank
	end
end

Baggins:RegisterSignal("Baggins_BagsUpdatedWhileOpen",
	function(self)
		self:RecheckCompress(compressLoopProtect>0 and 1.0 or 0.2);
	end,
	Baggins
);

Baggins:RegisterSignal("Baggins_BagOpened",
	function(self)
		self:RecheckCompress(0.1)
	end,
	Baggins
);



---------------------
-- ItemButton menu --
---------------------

Baggins:RegisterSignal("Baggins_ItemButtonMenu", 

	function(self, button, dewdrop, level, value)
		local bag = button:GetParent():GetID();
		local slot = button:GetID();
		local link = GetContainerItemLink(bag,slot)
		local itemstring = link:match("(item:[-%d:]+)");
	
		if level==1 then
			if not sieve:IsBankBag(bag) then
				dewdrop:AddLine("text",L["Use"], "secure", { type="item", item=itemstring },
					"closeWhenClicked",true, "tooltipText", L["Use/equip the item rather than bank/sell it"]);
				dewdrop:AddLine("text","","disabled",1);
			end
		end
	end,
	Baggins
);

